<?php
class BackendApi
{
  private $apiKey;
  private $options;
  private $backendUrl;
  private $pluginVersion;

  public function __construct($apiKey, $options, $backendUrl, $pluginVersion)
  {
    $this->apiKey = $apiKey;
    $this->options = $options;
    $this->backendUrl = $backendUrl;
    $this->pluginVersion = $pluginVersion;
  }

  public function getAccessToken()
  {
    $refreshToken = $this->options['refresh_token'];
    if (empty($refreshToken)) {
      throw new Exception('Refresh token is empty, go to mobile.de settings and logout + login again ');
    }

    $curl = curl_init();

    curl_setopt_array($curl, array(
      CURLOPT_URL => "https://securetoken.googleapis.com/v1/token?key=" . $this->apiKey,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => "",
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_TIMEOUT => 0,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => "POST",
      CURLOPT_POSTFIELDS => "grant_type=refresh_token&refresh_token=" . $refreshToken,
      CURLOPT_HTTPHEADER => array(
        "Content-Type: application/x-www-form-urlencoded"
      ),
    ));

    $response = curl_exec($curl);

    curl_close($curl);

    $retVal = json_decode($response);

    return $retVal->{'access_token'};
  }

  public function getData($shortCodeParams = "")
  {
    $refreshToken = $this->options['refresh_token'];
    $phoneNumber  = $this->options['phone_number'];
    $username = $this->options['username'];
    $pass = $this->options['password'];
    $customerId  = $this->options['customer_id'];
    $dataCredentials = $this->options['dataCredentials'];
    if (!isset($dataCredentials)) {
      $dataCredentials[] = [
        'clientNumber' => $customerId,
        'userName' => $username,
        'password' => $pass,
      ];
    }
    $mobileDeAuthList = array_map(function ($credential) {
      return (object)[
        'clientNumber' => $credential['clientNumber'],
        'mobileDeAuth' => base64_encode($credential['userName'] . ':' . $credential['password']),
      ];
    }, $dataCredentials);
    $domain = get_option('siteurl');

    $isDemoMode = empty($refreshToken);

    $accessToken = get_option('access_token');
    if (empty($accessToken) && !$isDemoMode) {
      $accessToken = $this->getAccessToken();
      add_option('access_token', $accessToken);
    }

    if (empty($accessToken) && !$isDemoMode) {
      throw new Exception('access token is empty, go to mobile.de settings and logout + login again');
    }

    $curl = curl_init();
    $apiBackendUrl = $this->backendUrl . "/api/cars";
    $data = [
      'ShortCodeParams' => base64_encode($shortCodeParams),
      'DataProvidersCredentials' => $mobileDeAuthList,
      'PhoneNumber' => $phoneNumber
    ];

    $jsonData = json_encode($data);

    // as of now WehikoPluginParams is mainly used for passing proper referer
    $WehikoPluginParams = "";
    if (isset($_POST['wehiko-plugin-params'])) {
      $WehikoPluginParams = $_POST['wehiko-plugin-params'];
    }
    $httpHeaders = array(
      "Domain: " . $domain,
      "PluginVersion: " . $this->pluginVersion,
      "WehikoPluginParams: " . $WehikoPluginParams,
      "Content-Type: application/json",
      "Content-Length: " . strlen($jsonData)
    );
    if (!$isDemoMode) {
      $httpHeaders = array(
        "Authorization: Bearer " . $accessToken,
        "Domain: " . $domain,
        "PluginVersion: " . $this->pluginVersion,
        "WehikoPluginParams: " . $WehikoPluginParams,
        "Content-Type: application/json",
        "Content-Length: " . strlen($jsonData)
      );
    }

    curl_setopt_array($curl, array(
      CURLOPT_URL => $apiBackendUrl,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_ENCODING => "",
      CURLOPT_MAXREDIRS => 10,
      CURLOPT_FOLLOWLOCATION => true,
      CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
      CURLOPT_CUSTOMREQUEST => "POST",
      CURLOPT_HTTPHEADER => $httpHeaders,
      CURLOPT_CONNECTTIMEOUT => 5,
      CURLOPT_TIMEOUT => 2 * 60,
      CURLOPT_POSTFIELDS => $jsonData,
    ));
    $result = curl_exec($curl);
    $curl_errno = curl_errno($curl);
    $curl_error = curl_error($curl);
    $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    curl_close($curl);

    $obj = new stdClass;
    $obj->status = $http_code;
    $obj->content = $result;
    $obj->errNo = $curl_errno;
    $obj->error = $curl_error;

    return $obj;
  }
}
