<?php
/*
 * Plugin Name: Wehiko.de
 * Plugin URI: https://wordpress-mobile.de
 * Description: Wordpress + Mobile.de integration plugin
 * Version: 2.4.0
 * Author: wehiko.de | Fewolino x Vanillia
 * Author URI: https://wordpress-mobile.de
 */
$apiKey = "AIzaSyCXS_1BR2H9-1KMUG3_iyKwx1cNVi-U-F8";
$configs = require_once('config.php');
require_once('backendApi.php');
$backendUrl = $configs['function-base-url'];
$environment = strpos($backendUrl, 'app.wehiko.eu') !== false ? "production" : "debug";
$isProd = $environment == "production";
$wehikoVehiclesFilePath = $_SERVER["DOCUMENT_ROOT"] . "/vehicles.html";
$optionsKey = 'dbi_mobile_de_plugin_options';

$plugin_path = plugin_dir_path(__FILE__);

$plugin_data = get_file_data($plugin_path . "wehiko.php", array('Version' => 'Version'), false);
$pluginVersion = $plugin_data['Version'];
$pluginSettingsBootstrapFile = $plugin_path . "index.html";

// REGISTER PLUGIN SETTINGS        
function mobile_de_plugin_register_settings()
{
  add_options_page('Wehiko', 'Wehiko', 'manage_options', 'dbi-mobile-de-plugin', 'dbi_render_plugin_settings_page');
}

function get_settings_view()
{
  global $isProd, $pluginSettingsBootstrapFile, $backendUrl;

  if (!$isProd) return;

  $cacheTime = 24 * 60 * 60; // 24 hours in seconds

  if (!file_exists($pluginSettingsBootstrapFile) || time() - filemtime($pluginSettingsBootstrapFile) >= $cacheTime) {
    // If the cache file doesn't exist or is older than 24 hours, download the HTML file and cache it      
    try {
      $result = getUrlContent($backendUrl . '/assets/plugin-settings/index.html');
      if ($result->status != 200) {
        echo "<br/>Error when initializing plugin settings<br/> Code: " . $result->status . "<br/>Message: " . $result->content;
      } else {
        $html = $result->content;
        if (!empty($html)) {
          file_put_contents($pluginSettingsBootstrapFile, $html);
        }
      }
    } catch (Exception $e) {
      echo "Error when initializing wehiko settings: " . $e->getMessage();
    }
  }
}

function getCachingPlugins()
{
  $all_plugins = get_plugins();
  $active_plugins = get_option('active_plugins');

  // Initialize an array to hold the data you want to collect
  $plugins_data = [];

  foreach ($active_plugins as $plugin_path) {
    // Extract the plugin's directory name as plain plugin name
    $plainPluginName = explode('/', $plugin_path)[0];

    // Assuming you want to fetch the plugin data based on the path
    if (isset($all_plugins[$plugin_path])) {
      $plugin_data = $all_plugins[$plugin_path];

      // You can add any other properties from $plugin_data or manipulate it as needed
      $plugins_data[] = [
        'Name' => $plugin_data['Name'] ?? 'N/A',
        'Version' => $plugin_data['Version'] ?? 'N/A',
        // Add any other data you're interested in
        'Directory' => $plainPluginName,
        'PluginURI' => $plugin_data['PluginURI'] ?? 'N/A',
      ];
    }
  }

  // For demonstration, let's encode this data as JSON
  return json_encode($plugins_data, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
}

function dbi_render_plugin_settings_page()
{
  global $backendUrl, $apiKey, $environment, $plugin_path, $pluginSettingsBootstrapFile,
    $pluginVersion, $cachingPlugins;

  $cachingPlugins = getCachingPlugins();

  require_once("initialData.php");

  get_settings_view();

  if (file_exists($pluginSettingsBootstrapFile)) {
    include_once($pluginSettingsBootstrapFile);
  } else {
    echo '<h1>Could not load plugin settings page, please send a screenshot of this page to our support email <a href="mailto:support@wordpress-mobile.de">support@wordpress-mobile.de</a></h1>';

    $active_plugins = get_option('active_plugins');
    $pluginListText = "";

    foreach ($active_plugins as $plugin) {
      $plainPluginName = explode('/', $plugin)[0];
      $pluginListText = $pluginListText . $plainPluginName . "<br/> ";
    }

    echo 'Plugin list:<br/>' . $pluginListText;
  }
}

add_action('admin_menu', 'mobile_de_plugin_register_settings');

function dbi_register_settings()
{
  global $optionsKey;
  register_setting($optionsKey, $optionsKey);
  add_settings_section('api_settings', '', '', 'dbi_mobile_de_plugin');
}

add_action('admin_init', 'dbi_register_settings');

function getPathForVehicle($hash)
{
  if (empty($hash)) {
    return $_SERVER["DOCUMENT_ROOT"] . '/vehicles';
  }

  return $_SERVER["DOCUMENT_ROOT"] . '/vehicles.' . $hash;
}

function wehiko_list_shortcode($atts)
{
  global $isProd, $wehikoVehiclesFilePath, $optionsKey;
  $currentVehiclesFilePath = $wehikoVehiclesFilePath;
  $customStyle = "";
  $customSettings = "";
  $hash = "";
  $options = get_option($optionsKey);
  $customJsCode = "<script type='text/javascript'>window.ajaxurl='" . admin_url('admin-ajax.php') . "';</script>";

  $has_attributes = !empty($atts) && is_array($atts);

  $json_attributes = json_encode($atts);
  if ($has_attributes) {
    /* echo 'Found short code attributes<br/>'; */
    $hash = md5($json_attributes);
    /* print_r($atts); */
  }

  if (!empty($hash)) {
    /* echo '<br/>Using hash: '.$hash; */
    $refreshingMarkerFile = 'wehiko-is-refreshing-data.' . $hash . '.txt';
    $currentVehiclesFilePath = getPathForVehicle($hash) . '.html';
    // Save the JSON attributes in a file
    file_put_contents(getPathForVehicle($hash) . '.json', $json_attributes);
  }

  $ajax_url = admin_url('admin-ajax.php?action=wehiko_refresh&hash=' . $hash);
  $customJsCode = "<script type='text/javascript'>window.ajaxurl='" . $ajax_url . "';</script>";
  if (isset($options['custom_settings_json'])) {
    $customSettingsJson = $options['custom_settings_json'];
    $customSettings = json_decode($customSettingsJson);

    if (is_object($customSettings)) {
      $customStyle = "<style>" .
        ".mobile-de-plugin.override .ad { background-color: " . $customSettings->bgColor . ";color:" . $customSettings->textColor . ";border-radius: " . $customSettings->borderRadius . "px}" .
        ".mobile-de-plugin.override .ad h2 a { color:" . $customSettings->titleTextColor . ";}" .
        ".mobile-de-plugin.override .ad .subtitle { color:" . $customSettings->subTitleTextColor . ";}" .
        ".mobile-de-plugin.override .ad .price { color:" . $customSettings->priceColor . ";}" .

        ".callMe.override .callToAction { background-color: " . $customSettings->bgIconColor . "; }" .
        ".callMe.override .callToAction svg { fill:" . $customSettings->iconColor . ";}" .
        ".callMe.override #phoneNoSlider { background-color:" . $customSettings->phoneNoBackgroundColor . ";color:" . $customSettings->phoneNoTextColor . ";border-radius: " . $customSettings->phoneNoBorderRadius . "px; }" .
        "</style>";
    }
  }

  if ($isProd) {
    $content = getVehicleFileContent($customJsCode . $customStyle, $customSettings, $hash);

    if (isset($content)) {
      return $content;
    }
  }

  get_vehicle_data(false, $hash);
  $isFileExisting = file_exists($currentVehiclesFilePath);
  $content = "no data, check plugin settings";
  if ($isFileExisting) {
    $content = file_get_contents($currentVehiclesFilePath);
    if (is_object($customSettings) && $customSettings->shouldMakeSlim) {
      $content = str_replace('div class="ads"', 'div class="ads make-slim"', $content);
    }
  }

  return $customJsCode . $customStyle . $content;
}

function getUrlContent($url)
{
  $curl = curl_init();

  curl_setopt($curl, CURLOPT_URL, $url);
  curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
  curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
  curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false); // You can remove this line if you want to verify SSL certificates

  $result = curl_exec($curl);

  $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
  curl_close($curl);

  $obj = new stdClass;
  $obj->status = $http_code;
  $obj->content = $result;

  return $obj;
}

// get local disk file or refresh it if wasn't refreshed for 24 hours
function getVehicleFileContent($customJsAndStyle, $customSettings, $hash)
{
  global $wehikoVehiclesFilePath;
  $currentVehiclesFilePath = $wehikoVehiclesFilePath;
  if (empty($currentVehiclesFilePath)) {
    return null;
  }
  if (!empty($hash)) {
    $currentVehiclesFilePath = getPathForVehicle($hash) . '.html';
  }
  $isFileExisting = file_exists($currentVehiclesFilePath);
  if ($isFileExisting) {
    $fileLastChangeDate = new DateTime();
    $fileLastChangeDate->setTimestamp(filemtime($currentVehiclesFilePath));

    $now = new DateTime();
    $now->setTimestamp(time());

    $secondsElapsed = $now->getTimestamp() - $fileLastChangeDate->getTimestamp();
    $minutes = 60;

    $secondsTriggeringRefresh = 24 * $minutes * 60;

    if ($secondsElapsed >= $secondsTriggeringRefresh) {
      get_vehicle_data(false, $hash);
    }
  } else {
    get_vehicle_data(false, $hash);
  }

  $content = file_get_contents($currentVehiclesFilePath);
  if (is_object($customSettings) && $customSettings->shouldMakeSlim) {
    $content = str_replace('div class="ads"', 'div class="ads make-slim"', $content);
  }
  return $customJsAndStyle . $content;
}

function log_error($message)
{
  $errorFilePath = __DIR__ . '/errors.html';
  $log_line = date_format(new DateTime(), "d.m.Y H:i:s") . " " . $message . "</br>";
  $isFileExisting = file_exists($errorFilePath);

  if ($isFileExisting) {
    $logTruncateSizeInKB = 10;
    $logSizeLimit = $logTruncateSizeInKB * 1024;
    $logContent = file_get_contents($errorFilePath);
    if (strlen($logContent) > $logSizeLimit) {
      $logContent = substr($logContent, 0, $logSizeLimit);
    }

    $log_line .= $logContent;
  }
  file_put_contents($errorFilePath, $log_line);
}

function get_vehicle_data($forceRefresh = false, $hash = "")
{
  global $apiKey, $optionsKey, $backendUrl, $wehikoVehiclesFilePath, $pluginVersion;
  $options = get_option($optionsKey);
  $mobileDataFetcher = new BackendApi($apiKey, $options, $backendUrl, $pluginVersion);
  $currentVehiclesFilePath = $wehikoVehiclesFilePath;
  $shortCodeParams = "";
  if (!empty($hash)) {
    $shortCodeParams = file_get_contents(getPathForVehicle($hash) . '.json');
    $currentVehiclesFilePath = getPathForVehicle($hash) . '.html';
  }

  $result = $mobileDataFetcher->getData($shortCodeParams);
  if ($result->errNo === CURLE_OPERATION_TIMEOUTED || $result->errNo === CURLE_COULDNT_CONNECT) {
    // try once more after first timeout
    $result = $mobileDataFetcher->getData($shortCodeParams);

    // if timed out second time then we don't try anymore
    if ($result->errNo === CURLE_OPERATION_TIMEOUTED || $result->errNo === CURLE_COULDNT_CONNECT) {
      log_error("Couldn't connect to server, timeout happened. Error code: " . $result->errNo . ", message: " . $result->error);
      return;
    }
  }

  //process result if there was no timeouts
  switch ($result->status) {
    case 401:
      $accessToken = $mobileDataFetcher->getAccessToken();
      update_option('access_token', $accessToken);
      if ($forceRefresh == false) {
        get_vehicle_data(true, $hash);
      }
      break;
    case 200:
      file_put_contents(getPathForVehicle($hash) . '.html', $result->content);
      break;
    default:
      $errorMessage = $result->content;
      if ($result->status == 0) {
        $errorMessage = "Can't connect to firebase";
      } else {
        log_error("Error status: " . $result->status);
      }
      log_error($errorMessage);
  }
}

add_shortcode('mobile_de_plugin', 'wehiko_list_shortcode');

add_action('init', 'activate_au');
function activate_au()
{
  global $pluginVersion;
  require_once('autoupdate.php');

  new mishaUpdateChecker($pluginVersion);
}

function wehiko_refresh()
{
  global $isProd, $wehikoVehiclesFilePath;
  $minutes = 60;

  // The $_REQUEST contains all the data sent via AJAX from the Javascript call
  if (isset($_REQUEST)) {
    $hash = $_GET['hash'] ?? '';
    $now = new DateTime();
    $now->setTimestamp(time());
    $refreshingMarkerFile = 'wehiko-is-refreshing-data.txt';
    $currentVehiclesFilePath = $wehikoVehiclesFilePath;

    if (!empty($hash)) {
      $refreshingMarkerFile = 'wehiko-is-refreshing-data.' . $hash . '.txt';
      $currentVehiclesFilePath = getPathForVehicle($hash) . '.html';
      echo 'Refreshing for hash: ' . $hash . PHP_EOL;
    }

    if (file_exists($refreshingMarkerFile)) {
      $markerFileSecondsTriggeringRefresh = 30 * $minutes;

      $markerFileLastChangeDate = new DateTime();
      $markerFileLastChangeDate->setTimestamp(filemtime($refreshingMarkerFile));
      $markerFileSecondsElapsed = $now->getTimestamp() - $markerFileLastChangeDate->getTimestamp();

      if ($markerFileSecondsElapsed < 0 || $markerFileSecondsElapsed >= $markerFileSecondsTriggeringRefresh) {
        echo 'sync takes to long, starting over' . PHP_EOL;
        unlink($refreshingMarkerFile);
      } else {
        echo 'refresh already in progress, it started at ' . date_format($markerFileLastChangeDate, 'Y-m-d H:i:s') . ' UTC' . PHP_EOL;
        die();
      }
    }

    $fileLastChangeDate = new DateTime();
    $secondsElapsed = 0;
    if (file_exists($currentVehiclesFilePath)) {
      $fileLastChangeDate->setTimestamp(filemtime($currentVehiclesFilePath));
      $secondsElapsed = $now->getTimestamp() - $fileLastChangeDate->getTimestamp();
    } else {
      echo '$wehikoVehiclesFilePath does not exist, force sync\n';
      $secondsElapsed = -1;
    }

    $secondsTriggeringRefresh = 0;
    if ($isProd) {
      $secondsTriggeringRefresh = 60 * $minutes;
      if ($secondsElapsed < 0) {
        if (file_exists($currentVehiclesFilePath)) {
          unlink($currentVehiclesFilePath);
        }
        $secondsElapsed = $secondsTriggeringRefresh; //enforce refresh
      }

      $nextRefreshInSeconds = $secondsTriggeringRefresh - $secondsElapsed;
      echo 'refreshes every: ' . ($secondsTriggeringRefresh / 60) . ' minutes | next refresh in: ' . floor($nextRefreshInSeconds / 60) . ' min ' . ($nextRefreshInSeconds % 60) . ' sec' . PHP_EOL;
    }
    // in dev environment refresh every time
    echo '$secondsElapsed: ' . $secondsElapsed . ', $secondsTriggeringRefresh: ' . $secondsTriggeringRefresh . PHP_EOL;
    if ($secondsElapsed >= $secondsTriggeringRefresh) {
      echo 'starting sync now' . PHP_EOL;
      touch($refreshingMarkerFile);
      get_vehicle_data(false, $hash);
      unlink($refreshingMarkerFile);
    }
    // Now let's return the result to the Javascript function (The Callback) 
    // for this echo should be used
  }

  // Always die in functions echoing AJAX content
  die();
}

function handle_wehiko_options()
{
  global $optionsKey, $wehikoVehiclesFilePath;
  switch ($_SERVER['REQUEST_METHOD']) {
    case 'GET':
      echo json_encode(get_option($optionsKey));
      /* $dataArray = get_option($optionsKey); */
      /* unset($dataArray['dataCredentials']); */
      /* echo json_encode($dataArray); */
      break;
    case 'DELETE':
      unlink($wehikoVehiclesFilePath);
      break;
    case 'POST':
      $newWehikoOptions = json_decode(stripslashes($_POST['data']), true);
      $wehikoOptionsInDb = get_option($optionsKey);

      $result = $newWehikoOptions;
      if (is_array($wehikoOptionsInDb)) {
        $result = array_replace($wehikoOptionsInDb, $newWehikoOptions);
      }
      echo 'data that will be saved: ' . var_dump($result);
      update_option($optionsKey, $result, false);
      break;
  }

  wp_die(); // this is required to terminate immediately and return a proper response        
}

// This bit is a special action hook that works with the WordPress AJAX functionality. 
add_action('wp_ajax_wehiko_refresh', 'wehiko_refresh');
add_action('wp_ajax_nopriv_wehiko_refresh', 'wehiko_refresh');

add_action('wp_ajax_wehiko_options', 'handle_wehiko_options');
